@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2002-2004 MySQL AB
+ Copyright (C) 2002-2005 MySQL AB
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of version 2 of the GNU General Public License as 
@@ -18,14 +18,13 @@
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-
-
+ 
  */
 package com.mysql.jdbc;
 
 import java.io.InputStream;
 import java.io.Reader;
+import java.io.UnsupportedEncodingException;
 
 import java.math.BigDecimal;
 
@@ -71,21 +70,25 @@ public class CallableStatement extends PreparedStatement implements
 		boolean isOut;
 
 		int jdbcType;
-		short nullability; 
+
+		short nullability;
+
 		String paramName;
-		int precision; 
-		int scale; 
+
+		int precision;
+
+		int scale;
+
 		String typeName;
-		
-		CallableStatementParam(String name, int idx, boolean in, 
-				boolean out, int jdbcType, String typeName, 
-				int precision, int scale, short nullability, 
-				int inOutModifier) {
+
+		CallableStatementParam(String name, int idx, boolean in, boolean out,
+				int jdbcType, String typeName, int precision, int scale,
+				short nullability, int inOutModifier) {
 			this.paramName = name;
 			this.isIn = in;
 			this.isOut = out;
 			this.index = idx;
-			
+
 			this.jdbcType = jdbcType;
 			this.typeName = typeName;
 			this.precision = precision;
@@ -104,7 +107,7 @@ public class CallableStatement extends PreparedStatement implements
 		}
 	}
 
-	class CallableStatementParamInfo implements ParameterMetaData {
+	class CallableStatementParamInfo {
 		String catalogInUse;
 
 		boolean isFunctionCall;
@@ -117,6 +120,42 @@ public class CallableStatement extends PreparedStatement implements
 
 		Map parameterMap;
 
+		/**
+		 * Constructor that converts a full list of parameter metadata into one
+		 * that only represents the placeholders present in the {CALL ()}.
+		 * 
+		 * @param fullParamInfo the metadata for all parameters for this stored 
+		 * procedure or function.
+		 */
+		CallableStatementParamInfo(CallableStatementParamInfo fullParamInfo) {
+			this.nativeSql = originalSql;
+			this.catalogInUse = currentCatalog;
+			isFunctionCall = fullParamInfo.isFunctionCall;
+			int[] localParameterMap = placeholderToParameterIndexMap;
+			int parameterMapLength = localParameterMap.length;
+			
+			parameterList = new ArrayList(fullParamInfo.numParameters);
+			parameterMap = new HashMap(fullParamInfo.numParameters);
+			
+			if (isFunctionCall) {
+				// Take the return value
+				parameterList.add(fullParamInfo.parameterList.get(0));
+			}
+			
+			int offset = isFunctionCall ? 1 : 0;
+			
+			for (int i = 0; i < parameterMapLength; i++) {
+				if (localParameterMap[i] != 0) {
+					CallableStatementParam param = (CallableStatementParam)fullParamInfo.parameterList.get(localParameterMap[i] + offset);
+					
+					parameterList.add(param);
+					parameterMap.put(param.paramName, param);
+				}
+			}
+			
+			this.numParameters = parameterList.size();
+		}
+		
 		CallableStatementParamInfo(java.sql.ResultSet paramTypesRs)
 				throws SQLException {
 			boolean hadRows = paramTypesRs.last();
@@ -137,17 +176,16 @@ public class CallableStatement extends PreparedStatement implements
 			} else {
 				this.numParameters = 0;
 			}
+			
+			if (isFunctionCall) {
+				this.numParameters += 1;
+			}
 		}
 
 		private void addParametersFromDBMD(java.sql.ResultSet paramTypesRs)
 				throws SQLException {
 			int i = 0;
 
-			if (isFunctionCall) {
-				// first row will be return value parameter
-				paramTypesRs.next();
-			}
-			
 			while (paramTypesRs.next()) {
 				String paramName = paramTypesRs.getString(4);
 				int inOutModifier = paramTypesRs.getInt(5);
@@ -155,7 +193,10 @@ public class CallableStatement extends PreparedStatement implements
 				boolean isOutParameter = false;
 				boolean isInParameter = false;
 
-				if (inOutModifier == DatabaseMetaData.procedureColumnInOut) {
+				if (i == 0 && isFunctionCall) {
+					isOutParameter = true;
+					isInParameter = false;
+				} else if (inOutModifier == DatabaseMetaData.procedureColumnInOut) {
 					isOutParameter = true;
 					isInParameter = true;
 				} else if (inOutModifier == DatabaseMetaData.procedureColumnIn) {
@@ -165,7 +206,7 @@ public class CallableStatement extends PreparedStatement implements
 					isOutParameter = true;
 					isInParameter = false;
 				}
-				
+
 				int jdbcType = paramTypesRs.getInt(6);
 				String typeName = paramTypesRs.getString(7);
 				int precision = paramTypesRs.getInt(8);
@@ -174,7 +215,7 @@ public class CallableStatement extends PreparedStatement implements
 
 				CallableStatementParam paramInfoToAdd = new CallableStatementParam(
 						paramName, i++, isInParameter, isOutParameter,
-						jdbcType, typeName, precision, scale, nullability, 
+						jdbcType, typeName, precision, scale, nullability,
 						inOutModifier);
 
 				this.parameterList.add(paramInfoToAdd);
@@ -185,8 +226,7 @@ public class CallableStatement extends PreparedStatement implements
 		protected void checkBounds(int paramIndex) throws SQLException {
 			int localParamIndex = paramIndex - 1;
 
-			if ((paramIndex < 0)
-					|| (localParamIndex >= this.numParameters)) {
+			if ((paramIndex < 0) || (localParamIndex >= this.numParameters)) {
 				throw new SQLException(
 						Messages.getString("CallableStatement.11") + paramIndex //$NON-NLS-1$
 								+ Messages.getString("CallableStatement.12") + numParameters //$NON-NLS-1$
@@ -213,47 +253,64 @@ public class CallableStatement extends PreparedStatement implements
 		}
 
 		public String getParameterClassName(int arg0) throws SQLException {
-			// TODO Auto-generated method stub
-			return null;
+			String mysqlTypeName = getParameterTypeName(arg0);
+			
+			boolean isBinaryOrBlob = StringUtils.indexOfIgnoreCase(mysqlTypeName, "BLOB") != -1 || 
+				StringUtils.indexOfIgnoreCase(mysqlTypeName, "BINARY") != -1;
+			
+			boolean isUnsigned = StringUtils.indexOfIgnoreCase(mysqlTypeName, "UNSIGNED") != -1;
+			
+			int mysqlTypeIfKnown = 0;
+			
+			if (StringUtils.startsWithIgnoreCase(mysqlTypeName, "MEDIUMINT")) {
+				mysqlTypeIfKnown = MysqlDefs.FIELD_TYPE_INT24;
+			}
+			
+			return ResultSetMetaData.getClassNameForJavaType(getParameterType(arg0), 
+					isUnsigned, mysqlTypeIfKnown, isBinaryOrBlob, false);
 		}
 
 		public int getParameterCount() throws SQLException {
+			if (this.parameterList == null) {
+				return 0;
+			}
+
 			return this.parameterList.size();
 		}
 
 		public int getParameterMode(int arg0) throws SQLException {
 			checkBounds(arg0);
-			
+
 			return getParameter(arg0 - 1).inOutModifier;
 		}
 
 		public int getParameterType(int arg0) throws SQLException {
 			checkBounds(arg0);
-			
+
 			return getParameter(arg0 - 1).jdbcType;
 		}
 
 		public String getParameterTypeName(int arg0) throws SQLException {
 			checkBounds(arg0);
-			
+
 			return getParameter(arg0 - 1).typeName;
 		}
 
 		public int getPrecision(int arg0) throws SQLException {
 			checkBounds(arg0);
-			
+
 			return getParameter(arg0 - 1).precision;
 		}
 
 		public int getScale(int arg0) throws SQLException {
 			checkBounds(arg0);
-			
+
 			return getParameter(arg0 - 1).scale;
 		}
 
 		public int isNullable(int arg0) throws SQLException {
 			checkBounds(arg0);
-			
+
 			return getParameter(arg0 - 1).nullability;
 		}
 
@@ -271,7 +328,26 @@ public class CallableStatement extends PreparedStatement implements
 			return this.numParameters;
 		}
 	}
-	
+
+	/**
+	 * Can't implement this directly, as then you can't use callable statements
+	 * on JDK-1.3.1, which unfortunately isn't EOL'd yet, and still present
+	 * quite a bit out there in the wild (Websphere, FreeBSD, anyone?)
+	 */
+
+	class CallableStatementParamInfoJDBC3 extends CallableStatementParamInfo
+			implements ParameterMetaData {
+
+		CallableStatementParamInfoJDBC3(java.sql.ResultSet paramTypesRs)
+				throws SQLException {
+			super(paramTypesRs);
+		}
+
+		public CallableStatementParamInfoJDBC3(CallableStatementParamInfo paramInfo) {
+			super(paramInfo);
+		}
+	}
+
 	private final static int NOT_OUTPUT_PARAMETER_INDICATOR = Integer.MIN_VALUE;
 
 	private final static String PARAMETER_NAMESPACE_PREFIX = "@com_mysql_jdbc_outparam_"; //$NON-NLS-1$
@@ -332,6 +408,10 @@ public class CallableStatement extends PreparedStatement implements
 
 		this.paramInfo = paramInfo;
 		this.callingStoredFunction = this.paramInfo.isFunctionCall;
+		
+		if (this.callingStoredFunction) {
+			this.parameterCount += 1;
+		}
 	}
 
 	/**
@@ -350,6 +430,65 @@ public class CallableStatement extends PreparedStatement implements
 		super(conn, catalog, null);
 
 		determineParameterTypes();
+		generateParameterMap();
+		
+		if (this.callingStoredFunction) {
+			this.parameterCount += 1;
+		}
+	}
+
+	private int[] placeholderToParameterIndexMap;
+	
+	
+	private void generateParameterMap() throws SQLException {
+		// if the user specified some parameters as literals, we need to
+		// provide a map from the specified placeholders to the actual
+		// parameter numbers
+		
+		int parameterCountFromMetaData = this.paramInfo.getParameterCount();
+		
+		// Ignore the first ? if this is a stored function, it doesn't count
+		
+		if (this.callingStoredFunction) {
+			parameterCountFromMetaData--;
+		}
+		
+		if (this.paramInfo != null &&
+				this.parameterCount != parameterCountFromMetaData) {
+			this.placeholderToParameterIndexMap = new int[this.parameterCount];
+			
+			int startPos = this.callingStoredFunction ? StringUtils.indexOfIgnoreCase(this.originalSql, 
+			"SELECT") : StringUtils.indexOfIgnoreCase(this.originalSql, "CALL");
+			
+			if (startPos != -1) {
+				int parenOpenPos = this.originalSql.indexOf('(', startPos + 4);
+				
+				if (parenOpenPos != -1) {
+					int parenClosePos = StringUtils.indexOfIgnoreCaseRespectQuotes(parenOpenPos, 
+							this.originalSql, ")", '\'', true);
+					
+					if (parenClosePos != -1) {
+						List parsedParameters = StringUtils.split(this.originalSql.substring(parenOpenPos + 1, parenClosePos), ",", "'\"", "'\"", true);
+						
+						int numParsedParameters = parsedParameters.size();
+						
+						// sanity check
+						
+						if (numParsedParameters != this.parameterCount) {
+							// bail?
+						}
+						
+						int placeholderCount = 0;
+						
+						for (int i = 0; i < numParsedParameters; i++) {
+							if (((String)parsedParameters.get(i)).equals("?")) {
+								this.placeholderToParameterIndexMap[placeholderCount++] = i;
+							}
+						}
+					}
+				}
+			}
+		}
 	}
 
 	/**
@@ -372,6 +511,11 @@ public class CallableStatement extends PreparedStatement implements
 		this.callingStoredFunction = isFunctionCall;
 
 		determineParameterTypes();
+		generateParameterMap();
+		
+		if (this.callingStoredFunction) {
+			this.parameterCount += 1;
+		}
 	}
 
 	/*
@@ -393,8 +537,8 @@ public class CallableStatement extends PreparedStatement implements
 
 				if (this.returnValueParam == null) {
 					this.returnValueParam = new CallableStatementParam("", 0,
-							false, true, Types.VARCHAR, "VARCHAR", 0, 0, 
-							DatabaseMetaData.attributeNullableUnknown, 
+							false, true, Types.VARCHAR, "VARCHAR", 0, 0,
+							DatabaseMetaData.attributeNullableUnknown,
 							DatabaseMetaData.procedureColumnReturn);
 				}
 
@@ -409,6 +553,10 @@ public class CallableStatement extends PreparedStatement implements
 
 		int localParamIndex = paramIndex - 1;
 
+		if (this.placeholderToParameterIndexMap != null) {
+			localParamIndex = this.placeholderToParameterIndexMap[localParamIndex];
+		}
+		
 		CallableStatementParam paramDescriptor = this.paramInfo
 				.getParameter(localParamIndex);
 
@@ -452,7 +600,7 @@ public class CallableStatement extends PreparedStatement implements
 
 	public synchronized void clearParameters() throws SQLException {
 		super.clearParameters();
-		
+
 		try {
 			if (this.outputParameterResults != null) {
 				this.outputParameterResults.close();
@@ -461,8 +609,9 @@ public class CallableStatement extends PreparedStatement implements
 			this.outputParameterResults = null;
 		}
 	}
-
+	
 	private void determineParameterTypes() throws SQLException {
+		
 		java.sql.ResultSet paramTypesRs = null;
 
 		try {
@@ -481,7 +630,7 @@ public class CallableStatement extends PreparedStatement implements
 					& useCatalog ? this.currentCatalog : null, null, procName,
 					"%"); //$NON-NLS-1$
 
-			this.paramInfo = new CallableStatementParamInfo(paramTypesRs);
+			convertGetProcedureColumnsToInternalDescriptors(paramTypesRs);
 		} finally {
 			SQLException sqlExRethrow = null;
 
@@ -501,6 +650,15 @@ public class CallableStatement extends PreparedStatement implements
 		}
 	}
 
+	private void convertGetProcedureColumnsToInternalDescriptors(java.sql.ResultSet paramTypesRs) throws SQLException {
+		if (!this.connection.isRunningOnJDK13()) {
+			this.paramInfo = new CallableStatementParamInfoJDBC3(
+					paramTypesRs);
+		} else {
+			this.paramInfo = new CallableStatementParamInfo(paramTypesRs);
+		}
+	}
+
 	/*
 	 * (non-Javadoc)
 	 * 
@@ -541,7 +699,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * 
 	 * @see java.sql.PreparedStatement#executeQuery()
 	 */
-	public synchronized java.sql.ResultSet executeQuery() throws SQLException {
+	public java.sql.ResultSet executeQuery() throws SQLException {
 		checkClosed();
 
 		checkStreamability();
@@ -565,7 +723,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * 
 	 * @see java.sql.PreparedStatement#executeUpdate()
 	 */
-	public synchronized int executeUpdate() throws SQLException {
+	public int executeUpdate() throws SQLException {
 		int returnVal = -1;
 
 		checkClosed();
@@ -645,15 +803,8 @@ public class CallableStatement extends PreparedStatement implements
 							? Messages.getString("CallableStatement.15") : Messages.getString("CallableStatement.16"), SQLError.SQL_STATE_ILLEGAL_ARGUMENT); //$NON-NLS-1$ //$NON-NLS-2$
 		}
 
+		
 		return mangleParameterName(paramNameIn);
-
-		/*
-		 * if (paramNameIn.startsWith("@")) { return paramNameIn; } else {
-		 * StringBuffer paramNameBuf = new StringBuffer("@");
-		 * paramNameBuf.append(paramNameIn);
-		 * 
-		 * return paramNameBuf.toString(); }
-		 */
 	}
 
 	/**
@@ -675,7 +826,7 @@ public class CallableStatement extends PreparedStatement implements
 	public synchronized Array getArray(String parameterName)
 			throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		Array retValue = rs.getArray(fixParameterName(parameterName));
 
@@ -733,7 +884,7 @@ public class CallableStatement extends PreparedStatement implements
 	public synchronized BigDecimal getBigDecimal(String parameterName)
 			throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		BigDecimal retValue = rs.getBigDecimal(fixParameterName(parameterName));
 
@@ -761,7 +912,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized Blob getBlob(String parameterName) throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		Blob retValue = rs.getBlob(fixParameterName(parameterName));
 
@@ -791,7 +942,7 @@ public class CallableStatement extends PreparedStatement implements
 	public synchronized boolean getBoolean(String parameterName)
 			throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		boolean retValue = rs.getBoolean(fixParameterName(parameterName));
 
@@ -819,7 +970,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized byte getByte(String parameterName) throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		byte retValue = rs.getByte(fixParameterName(parameterName));
 
@@ -848,7 +999,7 @@ public class CallableStatement extends PreparedStatement implements
 	public synchronized byte[] getBytes(String parameterName)
 			throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		byte[] retValue = rs.getBytes(fixParameterName(parameterName));
 
@@ -876,7 +1027,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized Clob getClob(String parameterName) throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		Clob retValue = rs.getClob(fixParameterName(parameterName));
 
@@ -919,7 +1070,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized Date getDate(String parameterName) throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		Date retValue = rs.getDate(fixParameterName(parameterName));
 
@@ -935,7 +1086,7 @@ public class CallableStatement extends PreparedStatement implements
 	public synchronized Date getDate(String parameterName, Calendar cal)
 			throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		Date retValue = rs.getDate(fixParameterName(parameterName), cal);
 
@@ -965,7 +1116,7 @@ public class CallableStatement extends PreparedStatement implements
 	public synchronized double getDouble(String parameterName)
 			throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		double retValue = rs.getDouble(fixParameterName(parameterName));
 
@@ -994,7 +1145,7 @@ public class CallableStatement extends PreparedStatement implements
 	public synchronized float getFloat(String parameterName)
 			throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		float retValue = rs.getFloat(fixParameterName(parameterName));
 
@@ -1022,7 +1173,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized int getInt(String parameterName) throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		int retValue = rs.getInt(fixParameterName(parameterName));
 
@@ -1050,7 +1201,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized long getLong(String parameterName) throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		long retValue = rs.getLong(fixParameterName(parameterName));
 
@@ -1060,14 +1211,15 @@ public class CallableStatement extends PreparedStatement implements
 	}
 
 	private int getNamedParamIndex(String paramName, boolean forOut)
-			throws SQLException {
+	throws SQLException {
+		
 		if ((paramName == null) || (paramName.length() == 0)) {
 			throw new SQLException(Messages.getString("CallableStatement.2"), //$NON-NLS-1$
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		}
 
 		CallableStatementParam namedParamInfo = this.paramInfo
-				.getParameter(paramName);
+		.getParameter(paramName);
 
 		if (this.paramInfo == null) {
 			throw new SQLException(
@@ -1078,11 +1230,23 @@ public class CallableStatement extends PreparedStatement implements
 		if (forOut && !namedParamInfo.isOut) {
 			throw new SQLException(
 					Messages.getString("CallableStatement.5") + paramName //$NON-NLS-1$
-							+ Messages.getString("CallableStatement.6"), //$NON-NLS-1$
+					+ Messages.getString("CallableStatement.6"), //$NON-NLS-1$
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		}
 
-		return namedParamInfo.index + 1; // JDBC indices are 1-based
+
+		if (this.placeholderToParameterIndexMap == null) {
+			return namedParamInfo.index + 1; // JDBC indices are 1-based
+		} 
+
+		for (int i = 0; i < this.placeholderToParameterIndexMap.length; i++) {
+			if (this.placeholderToParameterIndexMap[i] == namedParamInfo.index) {
+				return i + 1;
+			}
+		}
+
+		throw new SQLException("Can't find local placeholder mapping for parameter named \"" + 
+				paramName + "\".", SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 	}
 
 	/**
@@ -1124,7 +1288,7 @@ public class CallableStatement extends PreparedStatement implements
 	public synchronized Object getObject(String parameterName)
 			throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		Object retValue = rs.getObject(fixParameterName(parameterName));
 
@@ -1140,7 +1304,7 @@ public class CallableStatement extends PreparedStatement implements
 	public synchronized Object getObject(String parameterName, Map map)
 			throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		Object retValue = rs.getObject(fixParameterName(parameterName), map);
 
@@ -1181,6 +1345,15 @@ public class CallableStatement extends PreparedStatement implements
 
 	}
 
+	public synchronized ParameterMetaData getParameterMetaData()
+			throws SQLException {
+		if (this.placeholderToParameterIndexMap == null) {
+			return (CallableStatementParamInfoJDBC3) this.paramInfo;
+		} else {
+			return new CallableStatementParamInfoJDBC3(this.paramInfo);
+		}
+	}
+
 	/**
 	 * @see java.sql.CallableStatement#getRef(int)
 	 */
@@ -1200,7 +1373,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized Ref getRef(String parameterName) throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		Ref retValue = rs.getRef(fixParameterName(parameterName));
 
@@ -1229,7 +1402,7 @@ public class CallableStatement extends PreparedStatement implements
 	public synchronized short getShort(String parameterName)
 			throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		short retValue = rs.getShort(fixParameterName(parameterName));
 
@@ -1259,7 +1432,7 @@ public class CallableStatement extends PreparedStatement implements
 	public synchronized String getString(String parameterName)
 			throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		String retValue = rs.getString(fixParameterName(parameterName));
 
@@ -1302,7 +1475,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized Time getTime(String parameterName) throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		Time retValue = rs.getTime(fixParameterName(parameterName));
 
@@ -1318,7 +1491,7 @@ public class CallableStatement extends PreparedStatement implements
 	public synchronized Time getTime(String parameterName, Calendar cal)
 			throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		Time retValue = rs.getTime(fixParameterName(parameterName), cal);
 
@@ -1363,7 +1536,7 @@ public class CallableStatement extends PreparedStatement implements
 	public synchronized Timestamp getTimestamp(String parameterName)
 			throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		Timestamp retValue = rs.getTimestamp(fixParameterName(parameterName));
 
@@ -1379,7 +1552,7 @@ public class CallableStatement extends PreparedStatement implements
 	public synchronized Timestamp getTimestamp(String parameterName,
 			Calendar cal) throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		Timestamp retValue = rs.getTimestamp(fixParameterName(parameterName),
 				cal);
@@ -1408,7 +1581,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized URL getURL(String parameterName) throws SQLException {
 		ResultSet rs = getOutputParameters(0); // definitely not going to be
-												// from ?=
+		// from ?=
 
 		URL retValue = rs.getURL(fixParameterName(parameterName));
 
@@ -1428,6 +1601,10 @@ public class CallableStatement extends PreparedStatement implements
 
 		int localParamIndex = paramIndex - 1;
 
+		if (this.placeholderToParameterIndexMap != null) {
+			localParamIndex = this.placeholderToParameterIndexMap[localParamIndex];
+		}
+
 		int rsIndex = this.parameterIndexToRsIndex[localParamIndex];
 
 		if (rsIndex == NOT_OUTPUT_PARAMETER_INDICATOR) {
@@ -1689,7 +1866,7 @@ public class CallableStatement extends PreparedStatement implements
 								.clientPrepareStatement(queryBuf.toString());
 
 						byte[] parameterAsBytes = this
-								.getBytesRepresentation(parameterIndex);
+								.getBytesRepresentation(inParamInfo.index);
 
 						if (parameterAsBytes != null) {
 							if (parameterAsBytes.length > 8
@@ -1716,22 +1893,11 @@ public class CallableStatement extends PreparedStatement implements
 							setPstmt.close();
 						}
 					}
-
-					// StringBuffer fullName = new StringBuffer("@");
-					// fullName.append(outParameterName);
-					/*
-					 * this.setBytesNoEscapeNoQuotes(paramInfo.index + 1,
-					 * StringUtils.getBytes(outParameterName,
-					 * this.charConverter, this.charEncoding,
-					 * this.connection.getServerCharacterEncoding(),
-					 * this.connection.parserKnowsUnicode()));
-					 */
 				}
+
+				parameterIndex++;
 			}
-
-			parameterIndex++;
 		}
-
 	}
 
 	/**
@@ -1745,12 +1911,14 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#setLong(java.lang.String, long)
 	 */
 	public void setLong(String parameterName, long x) throws SQLException {
+		setLong(getNamedParamIndex(parameterName, false), x);
 	}
 
 	/**
 	 * @see java.sql.CallableStatement#setNull(java.lang.String, int)
 	 */
 	public void setNull(String parameterName, int sqlType) throws SQLException {
+		setNull(getNamedParamIndex(parameterName, false), sqlType);
 	}
 
 	/**
@@ -1759,6 +1927,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public void setNull(String parameterName, int sqlType, String typeName)
 			throws SQLException {
+		setNull(getNamedParamIndex(parameterName, false), sqlType, typeName);
 	}
 
 	/**
@@ -1793,10 +1962,18 @@ public class CallableStatement extends PreparedStatement implements
 				CallableStatementParam outParamInfo = (CallableStatementParam) paramIter
 						.next();
 
-				if (outParamInfo.isOut) {
+				if (!this.callingStoredFunction && outParamInfo.isOut) {
 					String outParameterName = mangleParameterName(outParamInfo.paramName);
 
-					this.setBytesNoEscapeNoQuotes(outParamInfo.index + 1,
+					int outParamIndex;
+					
+					if (this.placeholderToParameterIndexMap == null) { 
+							outParamIndex = outParamInfo.index + 1;
+					} else {
+							outParamIndex = this.placeholderToParameterIndexMap[outParamInfo.index - 1 /* JDBC is 1-based */];
+					}
+					
+					this.setBytesNoEscapeNoQuotes(outParamIndex,
 							StringUtils.getBytes(outParameterName,
 									this.charConverter, this.charEncoding,
 									this.connection
@@ -1869,4 +2046,21 @@ public class CallableStatement extends PreparedStatement implements
 	public synchronized boolean wasNull() throws SQLException {
 		return this.outputParamWasNull;
 	}
+
+	public int[] executeBatch() throws SQLException {
+		if (this.hasOutputParams) {
+			throw new SQLException("Can't call executeBatch() on CallableStatement with OUTPUT parameters",
+					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
+		}
+		
+		return super.executeBatch();
+	}
+
+	protected int getParameterIndexOffset() {
+		if (this.callingStoredFunction) {
+			return -1;
+		}
+		
+		return super.getParameterIndexOffset();
+	}
 }
