@@ -32,7 +32,7 @@ import java.sql.Types;
  * Field is a class used to describe fields in a ResultSet
  * 
  * @author Mark Matthews
- * @version $Id: Field.java 4297 2005-09-23 15:24:33Z mmatthews $
+ * @version $Id: Field.java 5639 2006-08-16 14:18:23Z mmatthews $
  */
 public class Field {
 	// ~ Static fields/initializers
@@ -113,6 +113,8 @@ public class Field {
 	
 	private boolean useOldNameMetadata = false;
 
+	private boolean isSingleBit;
+
 	// ~ Constructors
 	// -----------------------------------------------------------
 
@@ -217,7 +219,14 @@ public class Field {
 		}
 
 		if (this.mysqlType == MysqlDefs.FIELD_TYPE_BIT) {
-			if (this.length == 0) {
+			this.isSingleBit = (this.length == 0);
+			
+			if (this.connection != null && (this.connection.versionMeetsMinimum(5, 0, 21) ||
+					this.connection.versionMeetsMinimum(5, 1, 10)) && this.length == 1) {
+				this.isSingleBit = true;
+			}
+			
+			if (this.isSingleBit) {
 				this.sqlType = Types.BIT;
 			} else {
 				this.sqlType = Types.VARBINARY;
@@ -823,18 +832,37 @@ public class Field {
 		this.useOldNameMetadata = useOldNameMetadata;
 	}
 
-	/**
-	 * DOCUMENT ME!
-	 * 
-	 * @return DOCUMENT ME!
-	 */
 	public String toString() {
 		try {
-			return this.getDatabaseName()
-					+ " . " + this.getTableName() + "(" + this.getOriginalTableName() + ") . " + this.getName() + "(" + this.getOriginalName() + ")" + ", Mysql type: " + getMysqlType(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$
-		} catch (SQLException sqlEx) {
+			StringBuffer asString = new StringBuffer();
+			asString.append(super.toString());
+
+			asString.append("\n  catalog: ");
+			asString.append(this.getDatabaseName());
+			asString.append("\n  table name: ");
+			asString.append(this.getTableName());
+			asString.append("\n  original table name: ");
+			asString.append(this.getOriginalTableName());
+			asString.append("\n  column name: ");
+			asString.append(this.getName());
+			asString.append("\n  original column name: ");
+			asString.append(this.getOriginalName());
+			asString.append("\n  MySQL data type: ");
+			asString.append(getMysqlType());
+
+			if (this.buffer != null) {
+				asString.append("\n\nData as received from server:\n\n");
+				asString.append(StringUtils.dumpAsHex(this.buffer,
+						this.buffer.length));
+			}
+
+			return asString.toString();
+		} catch (Throwable t) {
 			return super.toString();
 		}
+	}
 
+	protected boolean isSingleBit() {
+		return this.isSingleBit;
 	}
 }
