@@ -337,6 +337,10 @@ public class ServerPreparedStatement extends PreparedStatement {
 
 	protected String asSql(boolean quoteStreamsAndUnknowns) throws SQLException {
 
+		if (this.isClosed) {
+			return "statement has been closed, no further internal information available";
+		}
+		
 		PreparedStatement pStmtForSub = null;
 
 		try {
@@ -629,7 +633,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 							} catch (SQLException ex) {
 								updateCounts[commandIndex] = EXECUTE_FAILED;
 
-								if (this.connection.getContinueBatchOnError()) {
+								if (this.continueBatchOnError) {
 									sqlEx = ex;
 								} else {
 									int[] newUpdateCounts = new int[commandIndex];
@@ -1473,8 +1477,18 @@ public class ServerPreparedStatement extends PreparedStatement {
 		if (x == null) {
 			setNull(parameterIndex, java.sql.Types.DECIMAL);
 		} else {
-			setString(parameterIndex, StringUtils.fixDecimalExponent(x
-					.toString()));
+		
+			BindValue binding = getBinding(parameterIndex, false);
+
+			if (this.connection.versionMeetsMinimum(5, 0, 3)) {
+				setType(binding, MysqlDefs.FIELD_TYPE_NEW_DECIMAL);
+			} else {
+				setType(binding, this.stringTypeCode);
+			}
+				
+			binding.value = StringUtils.fixDecimalExponent(x.toString());
+			binding.isNull = false;
+			binding.isLongData = false;
 		}
 	}
 
@@ -2031,6 +2045,8 @@ public class ServerPreparedStatement extends PreparedStatement {
 			case MysqlDefs.FIELD_TYPE_VAR_STRING:
 			case MysqlDefs.FIELD_TYPE_STRING:
 			case MysqlDefs.FIELD_TYPE_VARCHAR:
+			case MysqlDefs.FIELD_TYPE_DECIMAL:
+			case MysqlDefs.FIELD_TYPE_NEW_DECIMAL:
 				if (value instanceof byte[]) {
 					packet.writeLenBytes((byte[]) value);
 				} else if (!this.isLoadDataQuery) {
