@@ -25,6 +25,7 @@
 package com.mysql.jdbc;
 
 import java.sql.Date;
+import java.sql.SQLException;
 import java.sql.Time;
 import java.sql.Timestamp;
 
@@ -887,28 +888,61 @@ public class TimeUtil {
 	// WARN! You must externally synchronize these calendar instances
 	// See ResultSet.fastDateCreate() for an example
 	//
-	final static Date fastDateCreate(Calendar cal, int year, int month, int day) {
-		cal.clear();
+	final static Date fastDateCreate(boolean useGmtConversion,
+			Calendar gmtCalIfNeeded,
+			Calendar cal, int year, int month, int day) {
+		
+		Calendar dateCal = cal;
+		
+		if (useGmtConversion) {
+			
+			if (gmtCalIfNeeded == null) {
+				gmtCalIfNeeded = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
+			}
+			gmtCalIfNeeded.clear();
+			
+			dateCal = gmtCalIfNeeded;
+		}
+		
+		dateCal.clear();
 
 		// why-oh-why is this different than java.util.date,
 		// in the year part, but it still keeps the silly '0'
 		// for the start month????
-		cal.set(year, month - 1, day, 0, 0, 0);
-
+		dateCal.set(year, month - 1, day, 0, 0, 0);
+		
 		long dateAsMillis = 0;
 
 		try {
-			dateAsMillis = cal.getTimeInMillis();
+			dateAsMillis = dateCal.getTimeInMillis();
 		} catch (IllegalAccessError iae) {
 			// Must be on JDK-1.3.1 or older....
-			dateAsMillis = cal.getTime().getTime();
+			dateAsMillis = dateCal.getTime().getTime();
 		}
 
 		return new Date(dateAsMillis);
 	}
 
 	final static Time fastTimeCreate(Calendar cal, int hour, int minute,
-			int second) {
+			int second) throws SQLException {
+		if (hour < 0 || hour > 23) {
+			throw new SQLException("Illegal hour value '" + hour + "' for java.sql.Time type in value '"
+					+ timeFormattedString(hour, minute, second) + ".", 
+					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
+		}
+		
+		if (minute < 0 || minute > 59) {
+			throw new SQLException("Illegal minute value '" + minute + "'" + "' for java.sql.Time type in value '"
+					+ timeFormattedString(hour, minute, second) + ".", 
+					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
+		}
+		
+		if (second < 0 || second > 59) {
+			throw new SQLException("Illegal minute value '" + second + "'" + "' for java.sql.Time type in value '"
+					+ timeFormattedString(hour, minute, second) + ".", 
+					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
+		}
+		
 		cal.clear();
 
 		// Set 'date' to epoch of Jan 1, 1970
@@ -926,7 +960,9 @@ public class TimeUtil {
 		return new Time(timeAsMillis);
 	}
 
-	final static Timestamp fastTimestampCreate(Calendar cal, int year,
+	final static Timestamp fastTimestampCreate(boolean useGmtConversion,
+			Calendar gmtCalIfNeeded,
+			Calendar cal, int year,
 			int month, int day, int hour, int minute, int seconds,
 			int secondsPart) {
 		cal.clear();
@@ -936,6 +972,24 @@ public class TimeUtil {
 		// for the start month????
 		cal.set(year, month - 1, day, hour, minute, seconds);
 
+		int offsetDiff = 0;
+		
+		if (useGmtConversion) {
+			int fromOffset = cal.get(Calendar.ZONE_OFFSET)
+			+ cal.get(Calendar.DST_OFFSET);
+			
+			if (gmtCalIfNeeded == null) {
+				gmtCalIfNeeded = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
+			}
+			gmtCalIfNeeded.clear();
+			
+			gmtCalIfNeeded.setTimeInMillis(cal.getTimeInMillis());
+	
+			int toOffset = gmtCalIfNeeded.get(Calendar.ZONE_OFFSET)
+				+ gmtCalIfNeeded.get(Calendar.DST_OFFSET);
+			offsetDiff = fromOffset - toOffset;
+		}
+
 		long tsAsMillis = 0;
 
 		try {
@@ -945,7 +999,7 @@ public class TimeUtil {
 			tsAsMillis = cal.getTime().getTime();
 		}
 
-		Timestamp ts = new Timestamp(tsAsMillis);
+		Timestamp ts = new Timestamp(tsAsMillis + offsetDiff);
 		ts.setNanos(secondsPart);
 
 		return ts;
@@ -1021,4 +1075,34 @@ public class TimeUtil {
 
 		return canonicalTz;
 	}
+	
+	// we could use SimpleDateFormat, but it won't work when the time values
+	// are out-of-bounds, and we're using this for error messages for exactly 
+	// that case
+	//
+	
+	private static String timeFormattedString(int hours, int minutes, int seconds) {
+		StringBuffer buf = new StringBuffer(8);
+		if (hours < 10) {
+			buf.append("0");
+		}
+		
+		buf.append(hours);
+		buf.append(":");
+		
+		if (minutes < 10) {
+			buf.append("0");
+		}
+		
+		buf.append(minutes);
+		buf.append(":");
+		
+		if (seconds < 10) {
+			buf.append("0");
+		}
+		
+		buf.append(seconds);
+		
+		return buf.toString();
+	}
 }
