@@ -376,7 +376,7 @@ public class MetaDataRegressionTest extends BaseTestCase {
 							}
 						}
 
-						types.append(safeTypeName);
+						types.append(safeTypeName.toString());
 						types.append("Column ");
 						types.append(typeName);
 
@@ -813,6 +813,7 @@ public class MetaDataRegressionTest extends BaseTestCase {
 	 *             if the test fails.
 	 */
 	public void testBug7033() throws Exception {
+		if (false) { // disabled for now
 		Connection big5Conn = null;
 		Statement big5Stmt = null;
 		PreparedStatement big5PrepStmt = null;
@@ -858,6 +859,7 @@ public class MetaDataRegressionTest extends BaseTestCase {
 				big5Conn.close();
 			}
 		}
+		}
 	}
 
 	/**
@@ -1166,13 +1168,13 @@ public class MetaDataRegressionTest extends BaseTestCase {
 			assertEquals(false, dbmd.storesMixedCaseIdentifiers());
 			assertEquals(false, dbmd.storesMixedCaseQuotedIdentifiers());
 			assertEquals(false, dbmd.storesUpperCaseIdentifiers());
-			assertEquals(false, dbmd.storesUpperCaseQuotedIdentifiers());
+			assertEquals(true, dbmd.storesUpperCaseQuotedIdentifiers());
 		} else {
-			assertEquals(true, dbmd.storesLowerCaseIdentifiers());
-			assertEquals(true, dbmd.storesLowerCaseQuotedIdentifiers());
+			assertEquals(false, dbmd.storesLowerCaseIdentifiers());
+			assertEquals(false, dbmd.storesLowerCaseQuotedIdentifiers());
 			assertEquals(true, dbmd.storesMixedCaseIdentifiers());
 			assertEquals(true, dbmd.storesMixedCaseQuotedIdentifiers());
-			assertEquals(true, dbmd.storesUpperCaseIdentifiers());
+			assertEquals(false, dbmd.storesUpperCaseIdentifiers());
 			assertEquals(true, dbmd.storesUpperCaseQuotedIdentifiers());
 		}
 	}
@@ -1196,18 +1198,20 @@ public class MetaDataRegressionTest extends BaseTestCase {
 	    		  null, 
 	    		  "app tab");
 	    		  */
-	      rs = ((com.mysql.jdbc.DatabaseMetaData)this.conn.getMetaData()).extractForeignKeyFromCreateTable(this.conn.getCatalog(),"app tab");
-	     assertTrue("must return a row",rs.next()) ;
-	    
-	     assertEquals("comment; APPFK(`C1`) REFER `test`/ `app tab` (`C1`)", rs.getString(3));
+	      this.rs = ((com.mysql.jdbc.DatabaseMetaData)this.conn.getMetaData()).extractForeignKeyFromCreateTable(this.conn.getCatalog(),"app tab");
+	     assertTrue("must return a row",this.rs.next()) ;
 	     
-	     rs.close();
+	     String catalog = this.conn.getCatalog();
 	     
-	     rs = this.conn.getMetaData().getImportedKeys(this.conn.getCatalog(), null, "app tab");
+	     assertEquals("comment; APPFK(`C1`) REFER `" + catalog + "`/ `app tab` (`C1`)", this.rs.getString(3));
+	     
+	     this.rs.close();
+	     
+	     this.rs = this.conn.getMetaData().getImportedKeys(this.conn.getCatalog(), null, "app tab");
 	     
 	     assertTrue(this.rs.next());
 	     
-	     rs = this.conn.getMetaData().getExportedKeys(this.conn.getCatalog(), null, "app tab");
+	     this.rs = this.conn.getMetaData().getExportedKeys(this.conn.getCatalog(), null, "app tab");
 	     
 	     assertTrue(this.rs.next());
 	}
@@ -1250,6 +1254,34 @@ public class MetaDataRegressionTest extends BaseTestCase {
 		}
 	}
 	
+	/**
+	 * Tests fix for BUG#12975 - OpenOffice expects DBMD.supportsIEF()
+	 * to return "true" if foreign keys are supported by the datasource,
+	 * even though this method also covers support for check constraints,
+	 * which MySQL _doesn't_ have.
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug12975() throws Exception {
+		assertEquals(false, this.conn.getMetaData().supportsIntegrityEnhancementFacility());
+		
+		Connection overrideConn = null;
+		
+		try {
+			Properties props = new Properties();
+			
+			props.setProperty("overrideSupportsIntegrityEnhancementFacility",
+					"true");
+			
+			overrideConn = getConnectionWithProps(props);
+			assertEquals(true, overrideConn.getMetaData().supportsIntegrityEnhancementFacility());
+		} finally {
+			if (overrideConn != null) {
+				overrideConn.close();
+			}
+		}
+	}
+	
 	/**
 	 * Tests fix for BUG#13277 - RSMD for generated keys
 	 * has NPEs when a connection is referenced.
@@ -1257,6 +1289,10 @@ public class MetaDataRegressionTest extends BaseTestCase {
 	 * @throws Exception
 	 */
 	public void testBug13277() throws Exception {
+		if (isRunningOnJdk131()) {
+			return; // test not valid on JDK-1.3.1
+		}
+		
 		createTable("testBug13277", "(field1 INT NOT NULL PRIMARY KEY AUTO_INCREMENT, field2 VARCHAR(32))");
 		
 		try {
@@ -1316,11 +1352,205 @@ public class MetaDataRegressionTest extends BaseTestCase {
 		}
 	}
 	
+	/**
+	 * Tests BUG13601 (which doesn't seem to be present in 3.1.11,
+	 * but we'll leave it in here for regression's-sake).
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug13601() throws Exception {
+		
+		if (versionMeetsMinimum(5, 0)) {
+			createTable("testBug13601", "(field1 BIGINT NOT NULL, field2 BIT default 0 NOT NULL) ENGINE=MyISAM");
+			
+			this.rs = this.stmt.executeQuery("SELECT field1, field2 FROM testBug13601 WHERE 1=-1");
+			ResultSetMetaData rsmd = this.rs.getMetaData();
+			assertEquals(Types.BIT, rsmd.getColumnType(2));
+			assertEquals(Boolean.class.getName(), rsmd.getColumnClassName(2));
+			
+			this.rs = this.conn.prepareStatement("SELECT field1, field2 FROM testBug13601 WHERE 1=-1").executeQuery();
+			rsmd = this.rs.getMetaData();
+			assertEquals(Types.BIT, rsmd.getColumnType(2));
+			assertEquals(Boolean.class.getName(), rsmd.getColumnClassName(2));
+			
+		}
+	}
+	
+	/**
+	 * Tests fix for BUG#14815 - DBMD.getColumns() doesn't
+	 * return TABLE_NAME correctly.
+	 * 
+	 * @throws Exception if the test fails
+	 */
+	public void testBug14815() throws Exception {
+		try {
+			createTable("testBug14815_1", "(field_1_1 int)");
+			createTable("testBug14815_2", "(field_2_1 int)");
+			
+			boolean lcTableNames = 
+				this.conn.getMetaData().storesLowerCaseIdentifiers();
+			
+			String tableName1 = lcTableNames ? "testbug14815_1" : "testBug14815_1";
+			String tableName2 = lcTableNames ? "testbug14815_2" : "testBug14815_2";
+			
+			this.rs = this.conn.getMetaData().getColumns(
+					this.conn.getCatalog(), null, 
+					"testBug14815%", "%");
+			
+			assertTrue(this.rs.next());
+			assertEquals(tableName1, this.rs.getString("TABLE_NAME"));
+			assertEquals("field_1_1", this.rs.getString("COLUMN_NAME"));
+			
+			assertTrue(this.rs.next());
+			assertEquals(tableName2, this.rs.getString("TABLE_NAME"));
+			assertEquals("field_2_1", this.rs.getString("COLUMN_NAME"));
+			
+			
+		} finally {
+			if (this.rs != null) {
+				this.rs.close();
+				this.rs = null;
+			}
+		}
+	}
+	
+	/** 
+	 * Tests fix for BUG#15854 - DBMD.getColumns() returns wrong type
+	 * for BIT.
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug15854() throws Exception {
+		if (versionMeetsMinimum(5, 0)) {
+			createTable("testBug15854", "(field1 BIT)");
+			try {
+				this.rs = this.conn.getMetaData().getColumns(
+						this.conn.getCatalog(), null, "testBug15854", "field1");
+				assertTrue(this.rs.next());
+				assertEquals(Types.BIT, this.rs.getInt("DATA_TYPE"));
+			} finally {
+				if (this.rs != null) {
+					ResultSet toClose = this.rs;
+					this.rs = null;
+					toClose.close();
+				}
+			}
+			
+		}
+	}
+	
+	/**
+	 * Tests fix for BUG#16277 - Invalid classname returned for 
+	 * RSMD.getColumnClassName() for BIGINT type.
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug16277() throws Exception {
+		createTable("testBug16277", "(field1 BIGINT, field2 BIGINT UNSIGNED)");
+		ResultSetMetaData rsmd = this.stmt.executeQuery("SELECT field1, field2 FROM testBug16277").getMetaData();
+		assertEquals("java.lang.Long", rsmd.getColumnClassName(1));
+		assertEquals("java.math.BigInteger", rsmd.getColumnClassName(2));
+	}
+	
+	/**
+	 * Tests fix for BUG#18258 - Nonexistent catalog/database causes SQLException
+	 * to be raised, rather than returning empty result set.
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug18258() throws Exception {
+		String bogusDatabaseName = "abcdefghijklmnopqrstuvwxyz";
+		this.conn.getMetaData().getTables(bogusDatabaseName, "%", "%", new String[] {"TABLE", "VIEW"});
+		this.conn.getMetaData().getColumns(bogusDatabaseName, "%", "%", "%");
+		this.conn.getMetaData().getProcedures(bogusDatabaseName, "%", "%");
+	}
+	
+	/**
+	 * Tests fix for BUG#18554 - Aliased column names where
+	 * length of name > 251 are corrupted.
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug18554() throws Exception {
+		testBug18554(249);
+		testBug18554(250);
+		testBug18554(251);
+		testBug18554(252);
+		testBug18554(253);
+		testBug18554(254);
+		testBug18554(255);
+	}
+	
+	/*
+	public void testBug20491() throws Exception {
+		createTable("testBug20491",  "("
+				 + "`field1_ae_` int(10) unsigned NOT NULL default '0',"
+				 + "`field2_ue_` varchar(45) default '',"
+				 + "`field3_oe_` varchar(45) default '',"
+				 + "`field4_sz_` varchar(45) default '',"
+				 + "PRIMARY KEY  (`field1_ae_`))"
+				);
+		this.rs = this.conn.getMetaData().getColumns(this.conn.getCatalog(), 
+				null, "testBug20491", "%");
+		this.rs.next();
+		assertEquals("field1_ae_", this.rs.getString("COLUMN_NAME"));
+	}*/
+	
+	public void testBug20191() throws Exception {
+		if (versionMeetsMinimum(5, 0)) {
+			stmt.executeUpdate("drop table if exists t1;");
+		    stmt.executeUpdate("create table t1 (id int(10))");
+		    stmt.executeUpdate("insert into t1 values (1);");
+		    stmt.executeUpdate("drop view if exists v1");
+		    stmt.executeUpdate("CREATE ALGORITHM=UNDEFINED SQL SECURITY DEFINER VIEW v1 AS select t1.id as id from t1");
+		    ResultSet rs = stmt.executeQuery("select * from v1 group by id limit 1");
+		    System.out.println("when limit is 1, getTableName gives: ");
+		    ResultSetMetaData rsmd = rs.getMetaData();
+		    System.out.println(rsmd.getTableName(1));
+		    System.out.println("<P>");
+		    rs = stmt.executeQuery("select * from v1 group by id limit 0");
+		    System.out.println("when limit is 0, getTableName gives: ");
+		    rsmd = rs.getMetaData();
+		    System.out.println(rsmd.getTableName(1));
+		    System.out.println("<P>");
+		    rs = stmt.executeQuery("select * from v1 where id=1000 group by id ");
+		    System.out.println("when result not found, getTableName gives: ");
+		    rsmd = rs.getMetaData();
+		    System.out.println(rsmd.getTableName(1));
+		    System.out.println("<P>");
+		    rs = stmt.executeQuery("select * from v1 where id=1 group by id ");
+		    System.out.println("when result not found, getTableName gives: ");
+		    rsmd = rs.getMetaData();
+		    System.out.println(rsmd.getTableName(1));
+		    System.out.println("<P>");
+		    System.out.println("Note that \"group by\" is essential in this case");
+		    rs.close();
+		}
+	}
+	private void testBug18554(int columnNameLength) throws Exception {
+		StringBuffer buf = new StringBuffer(columnNameLength + 2);
+		
+		for (int i = 0; i < columnNameLength; i++) {
+			buf.append((char)((Math.random() * 26) + 65));
+		}
+		
+		
+		String colName = buf.toString();
+		this.rs = this.stmt.executeQuery("select curtime() as `" + colName + "`");
+						ResultSetMetaData meta = this.rs.getMetaData();
+
+		assertEquals(colName, meta.getColumnName(1));
+
+	}
+	
 	private void checkRsmdForBug13277(ResultSetMetaData rsmd) throws SQLException {
 		assertEquals(17, rsmd.getColumnDisplaySize(1));
-		assertEquals(false, rsmd.isDefinitelyWritable(1));
-		assertEquals(true, rsmd.isReadOnly(1));
-		assertEquals(false, rsmd.isWritable(1));
+		
+		if (versionMeetsMinimum(4, 1)) {
+			assertEquals(false, rsmd.isDefinitelyWritable(1));
+			assertEquals(true, rsmd.isReadOnly(1));
+			assertEquals(false, rsmd.isWritable(1));
+		}
 	}
 	
 	public void testSupportsCorrelatedSubqueries() throws Exception {
@@ -1335,6 +1565,16 @@ public class MetaDataRegressionTest extends BaseTestCase {
 		assertEquals(true, dbmd.supportsGroupByUnrelated());
 	}
 	
+	public void testRSMDToStringFromDBMD() throws Exception {
+		try {		
+			this.rs = this.conn.getMetaData().getTypeInfo();
+			
+			this.rs.getMetaData().toString(); // used to cause NPE
+		} finally {
+			closeMemberJDBCResources();
+		}
+	}
+	
 	private void testAbsenceOfMetadataForQuery(String query) throws Exception {
 		try {
 			this.pstmt = this.conn.prepareStatement(query);
