@@ -30,9 +30,12 @@ import java.io.CharArrayReader;
 import java.io.File;
 import java.io.FileOutputStream;
 import java.io.FileWriter;
+import java.io.IOException;
 import java.io.PrintStream;
+import java.io.StringReader;
 import java.io.Writer;
 import java.math.BigDecimal;
+import java.math.BigInteger;
 import java.sql.BatchUpdateException;
 import java.sql.Blob;
 import java.sql.Clob;
@@ -68,15 +71,15 @@ public class StatementRegressionTest extends BaseTestCase {
 		Connection c;
 
 		PrepareThread(Connection cn) {
-			c = cn;
+			this.c = cn;
 		}
 
 		public void run() {
 			for (int i = 0; i < 20; i++) // force this to end eventually
 			{
 				try {
-					c.prepareStatement("SELECT 1");
-					testServerPrepStmtDeadlockCounter++;
+					this.c.prepareStatement("SELECT 1");
+					StatementRegressionTest.this.testServerPrepStmtDeadlockCounter++;
 					Thread.sleep(400);
 				} catch (SQLException sqlEx) {
 					throw new RuntimeException(sqlEx);
@@ -341,10 +344,10 @@ public class StatementRegressionTest extends BaseTestCase {
 	 */
 	private void innerBug6823(boolean continueBatchOnError) throws SQLException {
 		Properties continueBatchOnErrorProps = new Properties();
-		continueBatchOnErrorProps.setProperty("continueBatchOnError", Boolean
-				.toString(continueBatchOnError));
+		continueBatchOnErrorProps.setProperty("continueBatchOnError", 
+				String.valueOf(continueBatchOnError));
 		this.conn = getConnectionWithProps(continueBatchOnErrorProps);
-		Statement statement = conn.createStatement();
+		Statement statement = this.conn.createStatement();
 
 		String tableName = "testBug6823";
 
@@ -398,7 +401,7 @@ public class StatementRegressionTest extends BaseTestCase {
 		int psRows = 0;
 		this.rs = this.stmt.executeQuery("SELECT * from " + tableName
 				+ " WHERE strdata1 like \"ps_%\"");
-		while (rs.next()) {
+		while (this.rs.next()) {
 			psRows++;
 		}
 		assertTrue(psRows > 0);
@@ -406,7 +409,7 @@ public class StatementRegressionTest extends BaseTestCase {
 		int sRows = 0;
 		this.rs = this.stmt.executeQuery("SELECT * from " + tableName
 				+ " WHERE strdata1 like \"s_%\"");
-		while (rs.next()) {
+		while (this.rs.next()) {
 			sRows++;
 		}
 		assertTrue(sRows > 0);
@@ -613,10 +616,14 @@ public class StatementRegressionTest extends BaseTestCase {
 	 *             if the test fails.
 	 */
 	public void testBug11798() throws Exception {
+		if (isRunningOnJdk131()) {
+			return; // test not valid on JDK-1.3.1
+		}
+		
 		try {
 			this.pstmt = this.conn.prepareStatement("SELECT ?");
 			this.pstmt.setObject(1, Boolean.TRUE, Types.BOOLEAN);
-			this.pstmt.setObject(1, new BigDecimal(1), Types.BOOLEAN);
+			this.pstmt.setObject(1, new BigDecimal("1"), Types.BOOLEAN);
 			this.pstmt.setObject(1, "true", Types.BOOLEAN);
 		} finally {
 			if (this.pstmt != null) {
@@ -625,7 +632,265 @@ public class StatementRegressionTest extends BaseTestCase {
 			}
 		}
 	}
+	
+	/** 
+	 * Tests fix for BUG#13255 - Reconnect during middle 
+	 * of executeBatch() should not happen.
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug13255() throws Exception {
+		
+		createTable("testBug13255", "(field_1 int)");
+		
+		Properties props = new Properties();
+		props.setProperty("autoReconnect", "true");
+		
+		Connection reconnectConn = null;
+		Statement reconnectStmt = null;
+		PreparedStatement reconnectPStmt = null;
+		
+		try {
+			reconnectConn = getConnectionWithProps(props);
+			reconnectStmt = reconnectConn.createStatement();
+			
+			String connectionId = getSingleIndexedValueWithQuery(reconnectConn, 1, "SELECT CONNECTION_ID()").toString();
+			
+			reconnectStmt.addBatch("INSERT INTO testBug13255 VALUES (1)");
+			reconnectStmt.addBatch("INSERT INTO testBug13255 VALUES (2)");
+			reconnectStmt.addBatch("KILL " + connectionId);
+			
+			for (int i = 0; i < 100; i++) {
+				reconnectStmt.addBatch("INSERT INTO testBug13255 VALUES (" + i + ")");	
+			}
+			
+			try {
+				reconnectStmt.executeBatch();
+			} catch (SQLException sqlEx) {
+				// We expect this...we killed the connection
+			}
+			
+			assertEquals(2, getRowCount("testBug13255"));
+			
+			this.stmt.executeUpdate("TRUNCATE TABLE testBug13255");
+			
+			reconnectConn.close();
+			
+			reconnectConn = getConnectionWithProps(props);
 
+			connectionId = getSingleIndexedValueWithQuery(reconnectConn, 1, "SELECT CONNECTION_ID()").toString();
+			
+			reconnectPStmt = reconnectConn.prepareStatement("INSERT INTO testBug13255 VALUES (?)");
+			reconnectPStmt.setInt(1, 1);
+			reconnectPStmt.addBatch();
+			reconnectPStmt.setInt(1, 2);
+			reconnectPStmt.addBatch();
+			reconnectPStmt.addBatch("KILL " + connectionId);
+			
+			for (int i = 3; i < 100; i++) {
+				reconnectPStmt.setInt(1, i);
+				reconnectPStmt.addBatch();
+			}
+			
+			try {
+				reconnectPStmt.executeBatch();
+			} catch (SQLException sqlEx) {
+				// We expect this...we killed the connection
+			}
+			
+			assertEquals(2, getRowCount("testBug13255"));
+			
+		} finally {
+			if (reconnectStmt != null) {
+				reconnectStmt.close();
+			}
+			
+			if (reconnectConn != null) {
+				reconnectConn.close();
+			}
+		}
+	}
+	
+	/**
+	 * Tests fix for BUG#15024 - Driver incorrectly closes streams passed as arguments
+	 * to PreparedStatements.
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug15024() throws Exception {
+		createTable("testBug15024", "(field1 BLOB)");
+		
+		try {
+			this.pstmt = this.conn.prepareStatement("INSERT INTO testBug15024 VALUES (?)");
+			testStreamsForBug15024(false, false);
+			
+			Properties props = new Properties();
+			props.setProperty("useConfigs", "3-0-Compat");
+			
+			Connection compatConn = null;
+			
+			try {
+				compatConn = getConnectionWithProps(props);
+
+				this.pstmt = compatConn.prepareStatement("INSERT INTO testBug15024 VALUES (?)");
+				testStreamsForBug15024(true, false);
+			} finally {
+				if (compatConn != null) {
+					compatConn.close();
+				}
+			}
+		} finally {
+			if (this.pstmt != null) {
+				PreparedStatement toClose = this.pstmt;
+				this.pstmt = null;
+				
+				toClose.close();
+			}
+		}
+	}
+
+	/**
+	 * PreparedStatement should call EscapeProcessor.escapeSQL?
+	 * 
+	 * @throws Exception if the test fails
+	 */
+	public void testBug15141() throws Exception {
+		try {
+			createTable("testBug15141", "(field1 VARCHAR(32))");
+			this.stmt.executeUpdate("INSERT INTO testBug15141 VALUES ('abc')");
+			
+			this.pstmt = this.conn.prepareStatement("select {d '1997-05-24'} FROM testBug15141");
+			this.rs = this.pstmt.executeQuery();
+			assertTrue(this.rs.next());
+			assertEquals("1997-05-24", this.rs.getString(1));
+			this.rs.close();
+			this.rs = null;
+			this.pstmt.close();
+			this.pstmt = null;
+			
+			this.pstmt = ((com.mysql.jdbc.Connection) this.conn).clientPrepareStatement("select {d '1997-05-24'} FROM testBug15141");
+			this.rs = this.pstmt.executeQuery();
+			assertTrue(this.rs.next());
+			assertEquals("1997-05-24", this.rs.getString(1));
+			this.rs.close();
+			this.rs = null;
+			this.pstmt.close();
+			this.pstmt = null;
+		} finally {
+			if (this.rs != null) {
+				ResultSet toCloseRs = this.rs;
+				this.rs = null;
+				toCloseRs.close();
+			}
+			
+			if (this.pstmt != null) {
+				PreparedStatement toClosePstmt = this.pstmt;
+				this.pstmt = null;
+				toClosePstmt.close();
+			}
+		}
+	}
+	
+	/**
+	 * Tests fix for BUG#18041 - Server-side prepared statements
+	 * don't cause truncation exceptions to be thrown.
+	 * 
+	 * @throws Exception if the test fails
+	 */
+	public void testBug18041() throws Exception {
+		if (versionMeetsMinimum(4, 1)) {
+			createTable("testBug18041", "(`a` tinyint(4) NOT NULL,"
+					+ "`b` char(4) default NULL)");
+	
+			Properties props = new Properties();
+			props.setProperty("jdbcCompliantTruncation", "true");
+			props.setProperty("useServerPrepStmts", "true");
+	
+			Connection truncConn = null;
+			PreparedStatement stm = null;
+			
+			try {
+				truncConn = getConnectionWithProps(props);
+
+				stm = truncConn
+					.prepareStatement("insert into testBug18041 values (?,?)");
+				stm.setInt(1, 1000);
+				stm.setString(2, "nnnnnnnnnnnnnnnnnnnnnnnnnnnnnn");
+				stm.executeUpdate();
+				fail("Truncation exception should have been thrown");
+			} catch (DataTruncation truncEx) {
+				// we expect this
+			} finally {
+				if (this.stmt != null) {
+					this.stmt.close();
+				}
+				
+				if (truncConn != null) {
+					truncConn.close();
+				}
+			}
+		}	
+	}
+	
+	private void testStreamsForBug15024(boolean shouldBeClosedStream, 
+			boolean shouldBeClosedReader) throws SQLException {
+		IsClosedInputStream bIn = new IsClosedInputStream(new byte[4]);
+		IsClosedReader readerIn = new IsClosedReader("abcdef");
+		
+		this.pstmt.setBinaryStream(1, bIn, 4);
+		this.pstmt.execute();
+		assertEquals(shouldBeClosedStream, bIn.isClosed());
+		
+		this.pstmt.setCharacterStream(1, readerIn, 6);
+		this.pstmt.execute();
+		assertEquals(shouldBeClosedReader, readerIn.isClosed());
+		
+		this.pstmt.close();
+	}
+    
+	class IsClosedReader extends StringReader {
+
+		boolean isClosed = false;
+		
+		public IsClosedReader(String arg0) {
+			super(arg0);
+		}
+
+		public void close() {
+			super.close();
+			
+			this.isClosed = true;
+		}
+		
+		public boolean isClosed() {
+			return this.isClosed;
+		}
+		
+	}
+	
+	class IsClosedInputStream extends ByteArrayInputStream {
+
+		boolean isClosed = false;
+		
+		public IsClosedInputStream(byte[] arg0, int arg1, int arg2) {
+			super(arg0, arg1, arg2);	
+		}
+
+		public IsClosedInputStream(byte[] arg0) {
+			super(arg0);
+		}
+
+		public void close() throws IOException {
+			// TODO Auto-generated method stub
+			super.close();
+			this.isClosed = true;
+		}
+		
+		public boolean isClosed() {
+			return this.isClosed;
+		}
+	}
+	
 	/**
 	 * Tests fix for BUG#1774 -- Truncated words after double quote
 	 * 
@@ -751,6 +1016,10 @@ public class StatementRegressionTest extends BaseTestCase {
 	 *             if the test fails
 	 */
 	public void testBug1934() throws Exception {
+		if (isRunningOnJdk131()) {
+			return; // test not valid on JDK-1.3.1
+		}
+		
 		try {
 			this.stmt.executeUpdate("DROP TABLE IF EXISTS testBug1934");
 			this.stmt.executeUpdate("CREATE TABLE testBug1934 (field1 INT)");
@@ -845,7 +1114,7 @@ public class StatementRegressionTest extends BaseTestCase {
 								+ " `field2` int(8) unsigned zerofill default NULL,"
 								+ " `field3` varchar(30) binary NOT NULL default '',"
 								+ " `field4` varchar(100) default NULL,"
-								+ " `field5` datetime NOT NULL default '0000-00-00 00:00:00',"
+								+ " `field5` datetime NULL default '0000-00-00 00:00:00',"
 								+ " PRIMARY KEY  (`field1`),"
 								+ " UNIQUE KEY `unq_id` (`field2`),"
 								+ " UNIQUE KEY  (`field3`),"
@@ -982,6 +1251,7 @@ public class StatementRegressionTest extends BaseTestCase {
 	 *             if the test fails.
 	 */
 	public void testBug3620() throws SQLException {
+		
 		long epsillon = 3000; // 3 seconds time difference
 
 		try {
@@ -1193,6 +1463,10 @@ public class StatementRegressionTest extends BaseTestCase {
 	 *             if the test fails
 	 */
 	public void testBug3873() throws Exception {
+		if (isRunningOnJdk131()) {
+			return; // test not valid on JDK-1.3.1
+		}
+		
 		PreparedStatement batchStmt = null;
 
 		try {
@@ -1313,6 +1587,10 @@ public class StatementRegressionTest extends BaseTestCase {
 	 *             if the test fails
 	 */
 	public void testBug4510() throws Exception {
+		if (isRunningOnJdk131()) {
+			return; // test not valid on JDK-1.3.1
+		}
+		
 		try {
 			this.stmt.executeUpdate("DROP TABLE IF EXISTS testBug4510");
 			this.stmt.executeUpdate("CREATE TABLE testBug4510 ("
@@ -1518,7 +1796,7 @@ public class StatementRegressionTest extends BaseTestCase {
 			this.rs = ps.executeQuery();
 
 			if (this.rs.next()) {
-				Date d = (Date) rs.getObject("field1");
+				Date d = (Date) this.rs.getObject("field1");
 				System.out.println("date: " + d);
 			}
 		} finally {
@@ -1681,18 +1959,18 @@ public class StatementRegressionTest extends BaseTestCase {
 			pstmt.setTime(2, nowTime);
 			pstmt.executeUpdate();
 
-			rs = tzStmt.executeQuery("SELECT * from timeTest");
+			this.rs = tzStmt.executeQuery("SELECT * from timeTest");
 
 			// Timestamps look like this: 2004-11-29 13:43:21
 			SimpleDateFormat timestampFormat = new SimpleDateFormat(
 					"yyyy-MM-dd HH:mm:ss");
 			SimpleDateFormat timeFormat = new SimpleDateFormat("HH:mm:ss");
 
-			while (rs.next()) {
+			while (this.rs.next()) {
 				// Driver now converts/checks DATE/TIME/TIMESTAMP/DATETIME types
 				// when calling getString()...
-				String retrTimestampString = new String(rs.getBytes(1));
-				Timestamp retrTimestamp = rs.getTimestamp(1);
+				String retrTimestampString = new String(this.rs.getBytes(1));
+				Timestamp retrTimestamp = this.rs.getTimestamp(1);
 
 				java.util.Date timestampOnServer = timestampFormat
 						.parse(retrTimestampString);
@@ -1706,8 +1984,8 @@ public class StatementRegressionTest extends BaseTestCase {
 								.abs(retrievedOffsetForTimestamp
 										- offsetDifference) < epsillon));
 
-				String retrTimeString = new String(rs.getBytes(2));
-				Time retrTime = rs.getTime(2);
+				String retrTimeString = new String(this.rs.getBytes(2));
+				Time retrTime = this.rs.getTime(2);
 
 				java.util.Date timeOnServerAsDate = timeFormat
 						.parse(retrTimeString);
@@ -1722,7 +2000,7 @@ public class StatementRegressionTest extends BaseTestCase {
 						(Math.abs(retrievedOffsetForTime - offsetDifference) < epsillon));
 			}
 		} finally {
-			stmt.executeUpdate("DROP TABLE IF EXISTS timeTest");
+			this.stmt.executeUpdate("DROP TABLE IF EXISTS timeTest");
 		}
 	}
 
@@ -1772,7 +2050,7 @@ public class StatementRegressionTest extends BaseTestCase {
 				this.pstmt.addBatch();
 			}
 
-			pstmt.executeBatch();
+			this.pstmt.executeBatch();
 
 		} finally {
 			this.stmt.executeUpdate("DROP TABLE IF EXISTS testBug8181");
@@ -1810,7 +2088,7 @@ public class StatementRegressionTest extends BaseTestCase {
 				while (this.rs.next())
 					;
 
-				rs.close();
+				this.rs.close();
 			}
 
 			if (this.pstmt != null) {
@@ -1902,7 +2180,10 @@ public class StatementRegressionTest extends BaseTestCase {
 	}
 
 	public void testCsc4194() throws Exception {
-
+		if (isRunningOnJdk131()) {
+			return; // test not valid on JDK-1.3.1
+		}
+		
 		Connection sjisConn = null;
 		Connection windows31JConn = null;
 
@@ -2001,6 +2282,10 @@ public class StatementRegressionTest extends BaseTestCase {
 	 *             if the test fails.
 	 */
 	public void testGetGeneratedKeysAllCases() throws Exception {
+		if (isRunningOnJdk131()) {
+			return; // test not valid on JDK-1.3.1
+		}
+		
 		System.out.println("Using Statement.executeUpdate()\n");
 
 		try {
@@ -2274,11 +2559,18 @@ public class StatementRegressionTest extends BaseTestCase {
 						.executeUpdate("CREATE TABLE testQuotedId (col1 VARCHAR(32))");
 
 				PreparedStatement pStmt = this.conn
-						.prepareStatement("SELECT * FROM testQuotedId FROM WHERE col1='ABC`DEF' or col1=?");
+						.prepareStatement("SELECT * FROM testQuotedId WHERE col1='ABC`DEF' or col1=?");
 				pStmt.setString(1, "foo");
-				System.out.println(pStmt);
+				pStmt.execute();
+				
+				this.stmt.executeUpdate("DROP TABLE IF EXISTS testQuotedId2");
+				this.stmt.executeUpdate("CREATE TABLE testQuotedId2 (`Works?` INT)");
+				pStmt = this.conn.prepareStatement("INSERT INTO testQuotedId2 (`Works?`) VALUES (?)");
+				pStmt.setInt(1, 1);
+				pStmt.executeUpdate();
 			} finally {
 				this.stmt.executeUpdate("DROP TABLE IF EXISTS testQuotedId");
+				this.stmt.executeUpdate("DROP TABLE IF EXISTS testQuotedId2");
 			}
 		}
 	}
@@ -2439,7 +2731,7 @@ public class StatementRegressionTest extends BaseTestCase {
 		testThread1.start();
 		testThread2.start();
 		Thread.sleep(30000);
-		assertTrue(testServerPrepStmtDeadlockCounter >= 10);
+		assertTrue(this.testServerPrepStmtDeadlockCounter >= 10);
 	}
 
 	/**
@@ -2697,4 +2989,418 @@ public class StatementRegressionTest extends BaseTestCase {
 		}
 	}
 
+	/**
+	 * Tests fix for BUG#15383 - PreparedStatement.setObject()
+	 * serializes BigInteger as object, rather than sending as
+	 * numeric value (and is thus not complementary to .getObject()
+	 * on an UNSIGNED LONG type).
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug15383() throws Exception {
+		createTable("testBug15383", "(id INTEGER UNSIGNED NOT NULL "
+				+ "AUTO_INCREMENT,value BIGINT UNSIGNED NULL DEFAULT 0,PRIMARY "
+				+ "KEY(id))ENGINE=InnoDB;");
+						
+		this.stmt.executeUpdate("INSERT INTO testBug15383(value) VALUES(1)");
+		
+		Statement updatableStmt = this.conn.createStatement(
+				ResultSet.TYPE_FORWARD_ONLY,
+				ResultSet.CONCUR_UPDATABLE);
+		
+		try {
+			this.rs = updatableStmt.executeQuery("SELECT * from testBug15383");
+			
+			assertTrue(this.rs.next());
+	
+			Object bigIntObj = this.rs.getObject("value");
+			assertEquals("java.math.BigInteger", bigIntObj.getClass().getName());
+			
+			this.rs.updateObject("value", new BigInteger("3"));
+			this.rs.updateRow();
+			
+			assertEquals("3", this.rs.getString("value"));
+		} finally {
+			if (this.rs != null) {
+				ResultSet toClose = this.rs;
+				this.rs = null;
+				toClose.close();
+			}
+			
+			if (updatableStmt != null) {
+				updatableStmt.close();
+			}
+		}		
+	}
+	
+	/**
+	 * Tests fix for BUG#17099 - Statement.getGeneratedKeys()
+	 * throws NPE when no query has been processed.
+	 * 
+	 * @throws Exception if the test fails
+	 */
+	public void testBug17099() throws Exception {
+		if (isRunningOnJdk131()) {
+			return; // test not valid
+		}
+		
+		Statement newStmt = this.conn.createStatement();
+		assertNotNull(newStmt.getGeneratedKeys());
+		
+		PreparedStatement pStmt = this.conn.prepareStatement("SELECT 1");
+		assertNotNull(pStmt.getGeneratedKeys());
+		
+		if (versionMeetsMinimum(4, 1)) {
+			pStmt = ((com.mysql.jdbc.Connection)this.conn).clientPrepareStatement("SELECT 1");
+			assertNotNull(pStmt.getGeneratedKeys());
+		}
+	}
+	
+	/**
+	 * Tests fix for BUG#17587 - clearParameters() on a closed prepared statement
+	 * causes NPE.
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug17587() throws Exception {
+		createTable("testBug17857", "(field1 int)");
+		PreparedStatement pStmt = null;
+		
+		try {
+			pStmt = this.conn.prepareStatement("INSERT INTO testBug17857 VALUES (?)");
+			pStmt.close();
+			try {
+				pStmt.clearParameters();
+			} catch (SQLException sqlEx) {
+				assertEquals("08003", sqlEx.getSQLState());
+			}
+			
+			pStmt = ((com.mysql.jdbc.Connection)this.conn).clientPrepareStatement("INSERT INTO testBug17857 VALUES (?)");
+			pStmt.close();
+			try {
+				pStmt.clearParameters();
+			} catch (SQLException sqlEx) {
+				assertEquals("08003", sqlEx.getSQLState());
+			}
+			
+		} finally {
+			if (pStmt != null) {
+				pStmt.close();
+			}
+		}
+	}
+	
+	/**
+	 * Tests fix for BUG#18740 - Data truncation and getWarnings()
+	 * only returns last warning in set.
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug18740() throws Exception {
+		if (!versionMeetsMinimum(5, 0, 2)) {
+			createTable("testWarnings", "(field1 smallint(6),"
+					+ "field2 varchar(6),"
+					+ "UNIQUE KEY field1(field1))");
+	
+			try {
+				this.stmt.executeUpdate("INSERT INTO testWarnings VALUES "
+					+ "(10001, 'data1'),"
+					+ "(10002, 'data2 foo'),"
+					+ "(10003, 'data3'),"
+					+ "(10004999, 'data4'),"
+					+ "(10005, 'data5')");
+			} catch (SQLException sqlEx) {
+				assertEquals("01004", sqlEx.getSQLState());
+				assertEquals("01004", sqlEx.getNextException().getSQLState());
+				
+				SQLWarning sqlWarn = this.stmt.getWarnings();
+				assertEquals("01000", sqlWarn.getSQLState());
+				assertEquals("01000", sqlWarn.getNextWarning().getSQLState());
+			}
+		}
+	}
+	
+	/**
+	 * Tests fix for BUG#19615, PreparedStatement.setObject(int, Object, int)
+	 * doesn't respect scale of BigDecimals.
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug19615() throws Exception {
+		createTable("testBug19615", "(field1 DECIMAL(19, 12))");
+		
+		try {
+			BigDecimal dec = new BigDecimal("1.234567");
+			
+			this.pstmt = this.conn.prepareStatement("INSERT INTO testBug19615 VALUES (?)");
+			this.pstmt.setObject(1, dec, Types.DECIMAL);
+			this.pstmt.executeUpdate();
+			this.pstmt.close();
+			
+			
+			this.rs = this.stmt.executeQuery("SELECT field1 FROM testBug19615");
+			this.rs.next();
+			assertEquals(dec, this.rs.getBigDecimal(1).setScale(6));
+			this.rs.close();
+			this.stmt.executeUpdate("TRUNCATE TABLE testBug19615");
+			
+			this.pstmt = ((com.mysql.jdbc.Connection)this.conn).clientPrepareStatement("INSERT INTO testBug19615 VALUES (?)");
+			this.pstmt.setObject(1, dec, Types.DECIMAL);
+			this.pstmt.executeUpdate();
+			this.pstmt.close();
+			
+			
+			this.rs = this.stmt.executeQuery("SELECT field1 FROM testBug19615");
+			this.rs.next();
+			assertEquals(dec, this.rs.getBigDecimal(1).setScale(6));
+			this.rs.close();
+		} finally {
+			closeMemberJDBCResources();
+		}
+		
+	}
+
+	public void testBug20029() throws Exception {
+		createTable("testBug20029", ("(field1 int)"));
+		
+		long initialTimeout = 20; // may need to raise this depending on environment
+		                          // we try and do this automatically in this testcase
+		
+		for (int i = 0; i < 10; i++) {
+			final Connection toBeKilledConn = getConnectionWithProps(new Properties());
+			final long timeout = initialTimeout;
+			PreparedStatement toBeKilledPstmt = null;
+			
+			try {
+				toBeKilledPstmt = ((com.mysql.jdbc.Connection)toBeKilledConn).clientPrepareStatement("INSERT INTO testBug20029 VALUES (?)");
+				
+				for (int j = 0; j < 1000; j++) {
+					toBeKilledPstmt.setInt(1, j);
+					toBeKilledPstmt.addBatch();
+				}
+				
+				Thread t = new Thread() {
+					public void run() {
+						try {
+							sleep(timeout);
+							toBeKilledConn.close();
+						} catch (Throwable t) {
+							
+						}
+					}
+				};
+				
+				t.start();
+				
+				try {
+					if (toBeKilledConn.isClosed()) {
+						initialTimeout *= 2;
+						continue;
+					}
+					
+					toBeKilledPstmt.executeBatch();
+					fail("Should've caught a SQLException for the statement being closed here");
+				} catch (BatchUpdateException batchEx) {
+					assertEquals("08003", batchEx.getSQLState());
+					break;
+				}
+				
+				fail("Connection didn't close while in the middle of PreparedStatement.executeBatch()");
+			} finally {
+				if (toBeKilledPstmt != null) {
+					toBeKilledPstmt.close();
+				}
+				
+				if (toBeKilledConn != null) {
+					toBeKilledConn.close();
+				}
+			}
+		}
+	}
+	
+	public void testLikeWithBackslashes() throws Exception {
+		if (!versionMeetsMinimum(5, 0, 0)) {
+			return;
+		}
+
+		Connection noBackslashEscapesConn = null;
+
+		try {
+			Properties props = new Properties();
+			props.setProperty("sessionVariables",
+					"sql_mode=NO_BACKSLASH_ESCAPES");
+
+			noBackslashEscapesConn = getConnectionWithProps(props);
+
+			createTable(
+					"X_TEST",
+					"(userName varchar(32) not null, ivalue integer, CNAME varchar(255), bvalue CHAR(1), svalue varchar(255), ACTIVE CHAR(1), primary key (userName))");
+
+			String insert_sql = "insert into X_TEST (ivalue, CNAME, bvalue, svalue, ACTIVE, userName) values (?, ?, ?, ?, ?, ?)";
+
+			this.pstmt = noBackslashEscapesConn.prepareStatement(insert_sql);
+			this.pstmt.setInt(1, 0);
+			this.pstmt.setString(2, "c:\\jetson");
+			this.pstmt.setInt(3, 1);
+			this.pstmt.setString(4, "c:\\jetson");
+			this.pstmt.setInt(5, 1);
+			this.pstmt.setString(6, "c:\\jetson");
+			this.pstmt.execute();
+
+			String select_sql = "select user0_.userName as userName0_0_, user0_.ivalue as ivalue0_0_, user0_.CNAME as CNAME0_0_, user0_.bvalue as bvalue0_0_, user0_.svalue as svalue0_0_, user0_.ACTIVE as ACTIVE0_0_ from X_TEST user0_ where user0_.userName like ?";
+			this.pstmt = noBackslashEscapesConn.prepareStatement(select_sql);
+			this.pstmt.setString(1, "c:\\j%");
+			// if we comment out the previous line and uncomment the following, the like clause matches
+			// stmt.setString(1,"c:\\\\j%");
+			System.out.println("about to execute query " + select_sql);
+			this.rs = this.pstmt.executeQuery();
+			assertTrue(this.rs.next());
+		} finally {
+			closeMemberJDBCResources();
+
+			if (noBackslashEscapesConn != null) {
+				noBackslashEscapesConn.close();
+			}
+		}
+	}
+
+	/**
+	 * Fixes BUG#20687 - Can't pool server-side prepared statements, exception
+	 * raised when re-using them.
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug20687() throws Exception {
+		if (!isRunningOnJdk131() && versionMeetsMinimum(4, 1)) {
+			createTable("testBug20687", "(field1 int)");
+			Connection poolingConn = null;
+			
+			Properties props = new Properties();
+			props.setProperty("cachePrepStmts", "true");
+			props.setProperty("prepStmtCacheSqlLimit", "5000");
+			
+			StringBuffer queryBuf = new StringBuffer("SELECT field1 FROM testBug20687 /*");
+			for (int i = 0; i < 1000; i++) {
+				queryBuf.append("a");
+			}
+			
+			queryBuf.append(" */");
+			
+			String query = queryBuf.toString();
+			
+			try {
+				poolingConn = getConnectionWithProps(props);
+				PreparedStatement pstmt1 = poolingConn.prepareStatement(query);
+				pstmt1.executeQuery();
+				pstmt1.close();
+				
+				PreparedStatement pstmt2 = poolingConn.prepareStatement(query);
+				pstmt2.executeQuery();
+				assertSame(pstmt1, pstmt2);
+				pstmt2.close();
+			} finally {
+				
+			}
+		}
+	}
+	
+	/**
+	 * Tests fix for BUG#20888 - escape of quotes in client-side prepared 
+	 * statements parsing not respected.
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug20888() throws Exception {
+		
+		try {
+			String s = "SELECT 'What do you think about D\\'Artanian''?', \"What do you think about D\\\"Artanian\"\"?\"";
+			this.pstmt = ((com.mysql.jdbc.Connection)this.conn).clientPrepareStatement(s);
+
+			this.rs = this.pstmt.executeQuery();
+			this.rs.next();
+			assertEquals(this.rs.getString(1), "What do you think about D'Artanian'?");
+			assertEquals(this.rs.getString(2), "What do you think about D\"Artanian\"?");
+		} finally {
+			closeMemberJDBCResources();
+		}
+	}
+	
+	/**
+	 * Tests Bug#21207 - Driver throws NPE when tracing prepared statements that
+	 * have been closed (in asSQL()).
+	 * 
+	 * @throws Exception if the test fails
+	 */
+	public void testBug21207() throws Exception {
+		try {
+			this.pstmt = this.conn.prepareStatement("SELECT 1");
+			this.pstmt.close();
+			this.pstmt.toString(); // this used to cause an NPE
+		} finally {
+			closeMemberJDBCResources();
+		}
+	}
+
+	/**
+	 * Tests fix for BUG#22290 - Driver issues truncation on write exception when
+	 * it shouldn't (due to sending big decimal incorrectly to server with
+	 * server-side prepared statement).
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug22290() throws Exception {
+		if (!versionMeetsMinimum(5, 0)) {
+			return;
+		}
+		
+		createTable(
+				"testbug22290",
+				"(`id` int(11) NOT NULL default '1',`cost` decimal(10,2) NOT NULL,PRIMARY KEY  (`id`)) ENGINE=InnoDB DEFAULT CHARSET=utf8");
+		assertEquals(
+				this.stmt
+						.executeUpdate("INSERT INTO testbug22290 (`id`,`cost`) VALUES (1,'1.00')"),
+				1);
+	
+		Connection configuredConn = null;
+		
+		try {
+			Properties props = new Properties();
+			props.setProperty("sessionVariables", "sql_mode='STRICT_TRANS_TABLES'");
+			
+			
+			configuredConn = getConnectionWithProps(props);
+			
+			this.pstmt = configuredConn
+					.prepareStatement("update testbug22290 set cost = cost + ? where id = 1");
+			this.pstmt.setBigDecimal(1, new BigDecimal("1.11"));
+			assertEquals(this.pstmt.executeUpdate(), 1);
+			
+			assertEquals(this.stmt
+					.executeUpdate("UPDATE testbug22290 SET cost='1.00'"), 1);
+			this.pstmt = ((com.mysql.jdbc.Connection)configuredConn)
+				.clientPrepareStatement("update testbug22290 set cost = cost + ? where id = 1");
+			this.pstmt.setBigDecimal(1, new BigDecimal("1.11"));
+			assertEquals(this.pstmt.executeUpdate(), 1);
+		} finally {
+			closeMemberJDBCResources();
+			
+			if (configuredConn != null) {
+				configuredConn.close();
+			}
+		}
+	}
+	
+	public void testClientPreparedSetBoolean() throws Exception {
+		try {
+			this.pstmt = ((com.mysql.jdbc.Connection)this.conn).clientPrepareStatement("SELECT ?");
+			this.pstmt.setBoolean(1, false);
+			assertEquals("SELECT 0", 
+					this.pstmt.toString().substring(this.pstmt.toString().indexOf("SELECT")));
+			this.pstmt.setBoolean(1, true);
+			assertEquals("SELECT 1", 
+					this.pstmt.toString().substring(this.pstmt.toString().indexOf("SELECT")));
+		} finally {
+			closeMemberJDBCResources();
+		}
+	}
 }
