@@ -24,28 +24,28 @@
  */
 package testsuite.simple;
 
-import testsuite.BaseTestCase;
-
 import java.sql.Connection;
 import java.sql.Date;
 import java.sql.PreparedStatement;
 import java.sql.SQLException;
 import java.sql.Statement;
+import java.sql.Time;
 import java.sql.Timestamp;
-
 import java.text.DateFormat;
 import java.text.SimpleDateFormat;
-
 import java.util.Calendar;
+import java.util.Locale;
 import java.util.Properties;
 import java.util.TimeZone;
 
+import testsuite.BaseTestCase;
+
 import com.mysql.jdbc.SQLError;
 
 /**
  * 
  * @author Mark Matthews
- * @version $Id: DateTest.java 3726 2005-05-19 15:52:24Z mmatthews $
+ * @version $Id: DateTest.java 5313 2006-05-30 17:59:29Z mmatthews $
  */
 public class DateTest extends BaseTestCase {
 	// ~ Constructors
@@ -252,24 +252,25 @@ public class DateTest extends BaseTestCase {
 					.executeQuery("SELECT fieldAsString, fieldAsDateTime FROM testZeroDateBehavior");
 			this.rs.next();
 
-			assertTrue("0001-01-01".equals(this.rs.getDate(1).toString()));
-			assertTrue("0001-01-01 00:00:00.0".equals(this.rs.getTimestamp(1)
-					.toString()));
-			assertTrue("0001-01-01".equals(this.rs.getDate(2).toString()));
-			assertTrue("0001-01-01 00:00:00.0".equals(this.rs.getTimestamp(2)
-					.toString()));
+			assertEquals("0001-01-01", this.rs.getDate(1).toString());
+			assertEquals("0001-01-01 00:00:00.0", 
+					new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.0", Locale.US).format(this.rs.getTimestamp(1)));
+			assertEquals("0001-01-01", this.rs.getDate(2).toString());
+			assertEquals("0001-01-01 00:00:00.0", 
+					new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.0", Locale.US).format(this.rs.getTimestamp(2)));
 
 			PreparedStatement roundPrepStmt = roundConn
 					.prepareStatement("SELECT fieldAsString, fieldAsDateTime FROM testZeroDateBehavior");
 			this.rs = roundPrepStmt.executeQuery();
 			this.rs.next();
 
-			assertTrue("0001-01-01".equals(this.rs.getDate(1).toString()));
-			assertTrue("0001-01-01 00:00:00.0".equals(this.rs.getTimestamp(1)
-					.toString()));
-			assertTrue("0001-01-01".equals(this.rs.getDate(2).toString()));
-			assertTrue("0001-01-01 00:00:00.0".equals(this.rs.getTimestamp(2)
-					.toString()));
+			assertEquals("0001-01-01", this.rs.getDate(1).toString());
+			assertEquals("0001-01-01 00:00:00.0", 
+					new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.0", Locale.US).format(this.rs.getTimestamp(1)));
+			assertEquals("0001-01-01", this.rs.getDate(2).toString());
+			assertEquals("0001-01-01 00:00:00.0", 
+					new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.0", Locale.US).format(this.rs.getTimestamp(2)));
+
 
 			props = new Properties();
 			props.setProperty("zeroDateTimeBehavior", "convertToNull");
@@ -378,4 +379,39 @@ public class DateTest extends BaseTestCase {
 			this.stmt.executeUpdate("DROP TABLE IF EXISTS testReggieBug");
 		}
 	}
+	
+	public void testNativeConversions() throws Exception {
+		Timestamp ts = new Timestamp(System.currentTimeMillis());
+		Date dt = new Date(ts.getTime());
+		Time tm = new Time(ts.getTime());
+		
+		createTable("testNativeConversions", "(time_field TIME, date_field DATE, datetime_field DATETIME, timestamp_field TIMESTAMP)");
+		this.pstmt = this.conn.prepareStatement("INSERT INTO testNativeConversions VALUES (?,?,?,?)");
+		this.pstmt.setTime(1, tm);
+		this.pstmt.setDate(2, dt);
+		this.pstmt.setTimestamp(3, ts);
+		this.pstmt.setTimestamp(4, ts);
+		this.pstmt.execute();
+		this.pstmt.close();
+		
+		this.pstmt = this.conn.prepareStatement("SELECT time_field, date_field, datetime_field, timestamp_field FROM testNativeConversions");
+		this.rs = this.pstmt.executeQuery();
+		assertTrue(this.rs.next());
+		System.out.println(this.rs.getTime(1));
+		System.out.println(this.rs.getTime(2));
+		System.out.println(this.rs.getTime(3));
+		System.out.println(this.rs.getTime(4));
+		System.out.println();
+		System.out.println(this.rs.getDate(1));
+		System.out.println(this.rs.getDate(2));
+		System.out.println(this.rs.getDate(3));
+		System.out.println(this.rs.getDate(4));
+		System.out.println();
+		System.out.println(this.rs.getTimestamp(1));
+		System.out.println(this.rs.getTimestamp(2));
+		System.out.println(this.rs.getTimestamp(3));
+		System.out.println(this.rs.getTimestamp(4));
+		
+		
+	}
 }
